/*
 * GPRMCEvent.java
 * Recommended Minimum Specific GPS/TRANSIT Data
 *
 * Created on May 30, 2004, 1:12 PM
 */

import java.util.GregorianCalendar;
import java.util.Date;
import java.util.TimeZone;
import java.text.NumberFormat;

public class GPRMCEvent {
    //This is a conglomeration of POS_UTC and DATE
    public long dateTime; // Same as currentTimeMillis
    
    public boolean signalAcquired; // true==A, false==V
    public double latitudeDecimalDegrees; // Receiver gives ddmm.mmmm
    public double longitudeDecimalDegrees; // Receiver gives dddmm.mmmm
    public double speedOverGround; // 0.00 (knots)
    public double headingOverGround; // (degrees)
    public double magneticVariation; // (degrees)
    public double timeDbl; // (globaltime)
    public boolean magneticVariationPresent;
    
    /** Creates a new instance of GPRMCEvent */
    public GPRMCEvent() {
    }
    
    public String toString() {
        if (signalAcquired == false) {
            return "Invalid";
        }
        
        Date date = new Date(dateTime);
        String result = new String(date
        + " Lat: " + ((double)((int)(latitudeDecimalDegrees*100000))/100000)
        + " Lon: " + ((double)((int)(longitudeDecimalDegrees*100000))/100000)
        + " Speed: " + speedOverGround
        + " Heading: " + headingOverGround);
        if (magneticVariationPresent == true) {
            result += (" MagVar: " + magneticVariation);
        }
        return result;
    }
    
    public GPRMCEvent(String gprmcString) {
        //String.split only present after JDK 1.4
        //The reg ex is for either , or *
        String[] fields = gprmcString.split("[,[\u002A-\u002A]]");
        
        if (!fields[0].equalsIgnoreCase("$GPRMC")) {
            throw new IllegalArgumentException("Invalid GPRMC Sentence: "
            + "\n" + gprmcString
            + "\nIt did not begin with \"$GPRMC\".");
        }
        
        // Valid
        String validString = fields[2];
        if (validString.equalsIgnoreCase("A")) {
            signalAcquired = true;
        } else {
            //Could do assert check if "V" here.
            return;
        }
        
        // Date & Time
        timeDbl = Double.parseDouble(fields[1]);
        String dateString = fields[9];
        String timeString = fields[1];
        if (dateString.length() >= 6) {
            int year =  Integer.parseInt(dateString.substring(4,6)) + 2000;
            int month = Integer.parseInt(dateString.substring(2,4)) - 1;
            int day =   Integer.parseInt(dateString.substring(0,2));
            int hrs =   Integer.parseInt(timeString.substring(0,2));
            int min =   Integer.parseInt(timeString.substring(2,4));
            int sec =   Integer.parseInt(timeString.substring(4,6));
            
            GregorianCalendar cal =
            new GregorianCalendar(year, month, day, hrs, min, sec);
            cal.setTimeZone(TimeZone.getTimeZone("UTC"));
            dateTime = cal.getTimeInMillis();
            
            //If string contained milliseconds, add them
            int decimalPos = timeString.indexOf(".");
            if (decimalPos >= 0) {
                dateTime +=
                Integer.parseInt(timeString.substring(decimalPos+1));
            }
        }

        NumberFormat f = NumberFormat.getNumberInstance();
	f.setMaximumFractionDigits(7);

        //Latitude
        String latStr = fields[3];
        latitudeDecimalDegrees =
        Double.parseDouble(latStr.substring(0,2))
        + (Double.parseDouble(latStr.substring(2)) / 60);
        
        if (fields[4].equalsIgnoreCase("S")) {
            latitudeDecimalDegrees = -latitudeDecimalDegrees;
        }
        
        //Longitude
        String longStr = fields[5];
        longitudeDecimalDegrees =
        Double.parseDouble(longStr.substring(0,3))
        + (Double.parseDouble(longStr.substring(3)) / 60);
        
        if (fields[6].equalsIgnoreCase("W")) {
            longitudeDecimalDegrees = -longitudeDecimalDegrees;
        }
        
        //Speed
        speedOverGround = Double.parseDouble(fields[7]);
        
        //Heading
        headingOverGround = Double.parseDouble(fields[8]);
        
        //Magnetic variation
        String field10 = fields[10];
        if (fields[10].length() != 0) { //Is the field present?
            magneticVariationPresent = true;
            magneticVariation = Double.parseDouble(fields[10]);
            if (fields[11].equalsIgnoreCase("W")) {
                magneticVariation = -magneticVariation;
            }
            
        }
    }
    
    public static void main(String args[]) {
        GPRMCEvent e = new GPRMCEvent(
        "GPRMC,092204.999,A,4250.5589,S,14718.5084,E,0.00,89.68,211200,,*25");
        System.out.println(e);
        
        e = new GPRMCEvent(
        "GPRMC,195106,A,2806.6040,N,08223.5265,W,000.0,099.3,190604,004.6,W*76");
        System.out.println(e);
        
        e = new GPRMCEvent(
        "GPRMC,,V,2806.6001,N,08223.5356,");
        System.out.println(e);        
        
        e = new GPRMCEvent(
        "GPRMC,215806,V,2806.6048,N,08223.5259,W,,,190604,004.6,W*67");
        System.out.println(e); 
    }
    
}
